#!/usr/bin/env python
# -*- coding: utf-8 -*-
# academic_german_wiki.py
"""Collection of German academic degrees listed in Wikipedia."""
# import itertools
import requests
import unicodedata

from bs4 import BeautifulSoup


def degrees_ger_wiki() -> list:
    try:
        with open('./persontitles/data/academic_german_wiki.txt', mode='r', encoding='utf-8') as fin:  # noqa
            DEGREES = fin.read().split('\n')
    except FileNotFoundError:
        DEGREES = _degrees()
        with open('./persontitles/data/academic_german_wiki.txt', mode='a', encoding='utf-8') as fout:  # noqa
            fout.write('\n'.join(item for item in DEGREES))

    return DEGREES


def get_lines(soup) -> list:
    lines = []
    for tr in soup.find_all('tr'):
        line = [td.text for td in tr.find_all('td')]
        try:
            if line[0].startswith('...'):
                pass
            else:
                lines.append(line[0].strip())
        except IndexError:
            # print("IndexError:", values)
            pass
    return lines


def split_lines(lines) -> list:
    lines_with_added_splits = []

    for line in lines:
        line = line.strip()
        if '/Kunstvermittlung' in line:
            lines_with_added_splits.append(line)
        elif '/' in line:
            splits = line.split('/')
            for split in splits:
                lines_with_added_splits.append(split.strip())
        elif 'oder' in line:
            splits = line.split('oder')
            for split in splits:
                lines_with_added_splits.append(split.strip())
        elif 'bzw.' in line:
            splits = line.split('bzw.')
            for split in splits:
                lines_with_added_splits.append(split.strip())
        else:
            lines_with_added_splits.append(line)

    return lines_with_added_splits


def declutter_lines(lines) -> list:
    lines_wo_clutter = []

    for line in lines:
        if line.endswith(']'):
            line = line.split('[')[0].strip()
            lines_wo_clutter.append(line)
        elif line.endswith(')'):
            bracket_content = line.split('(')[-1]
            if bracket_content.startswith('M') and bracket_content != 'Med.)':
                lines_wo_clutter.append(bracket_content[:-1])
            else:
                lines_wo_clutter.append(line)
        else:
            lines_wo_clutter.append(line)

    return lines_wo_clutter


def lines_wo_brackets(lines) -> list:
    lines_wo_brackets = []

    for line in lines:
        splits = line.split('(')
        if len(splits) > 1:
            bracket_content = splits[-1].strip()
            if bracket_content[:-1] in ['FH', 'DH', 't.o.', 'Med.']:
                lines_wo_brackets.append(line)
            elif '.' not in bracket_content:
                lines_wo_brackets.append(splits[0].strip())
            elif bracket_content.endswith('Dr.)'):
                lines_wo_brackets.append(splits[0].strip())
                lines_wo_brackets.append(bracket_content[:-1])
            else:
                lines_wo_brackets.append(splits[0].strip())
                lines_wo_brackets.append(bracket_content)
        elif line.endswith('])'):
            pass
        else:
            lines_wo_brackets.append(line)

    return lines_wo_brackets


def final_degrees(lines_wo_brackets) -> list:
    lines_ = lines_wo_brackets
    lines_wo_brackets = []
    for line in lines_:
        if '(' in line:
            lines_wo_brackets.append(line)
        # because there is no bracket pair (otherwise len(splits) would have
        # been greater than 1, it is possible to look for solitary brackets
        elif line.endswith(')'):
            if 'altertümlich' in line:
                pass
            else:
                lines_wo_brackets.append(line[:-1])
        else:
            lines_wo_brackets.append(line)

    return lines_wo_brackets


def normalize_degrees(degrees) -> list:
    normalized_degrees = []
    for degree in set(degrees):
        degree = unicodedata.normalize('NFKD', degree)
        normalized_degrees.append(degree)

    return normalized_degrees


def _degrees() -> list:
    data = requests.get('https://de.wikipedia.org/wiki/Liste_akademischer_Grade_(Deutschland)')  # noqa
    soup = BeautifulSoup(data.text, 'lxml')
    lines = get_lines(soup)
    lines = split_lines(lines)
    lines = declutter_lines(lines)
    degrees = lines_wo_brackets(lines)
    degrees = final_degrees(degrees)
    normalized_degrees = normalize_degrees(degrees)

    return normalized_degrees


if __name__ == '__main__':
    ACADEMIC = degrees_ger_wiki()
    for i, degree in enumerate(sorted(ACADEMIC)):
        print(i, degree)



# -*- coding: utf-8 -*-
# context.py
import os
import sys

PACKAGE_PARENT = '..'
SCRIPT_DIR = os.path.dirname(
    os.path.realpath(os.path.join(os.getcwd(), os.path.expanduser(__file__))),
)  # isort:skip # noqa # pylint: disable=wrong-import-position
sys.path.append(
    os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)),
)  # isort: skip # noqa # pylint: disable=wrong-import-position

from persontitles import (  # type: ignore # isort:skip # noqa # pylint: disable=unused-import, wrong-import-position
    academic_german,
    academic_german_wiki,
    academic_german_drtitel,
    academic_uk,
    academic_us,
    academic_degrees,
    peertitles,
    job_titles,
    gov_jobs,
)  # pylint: disable=unused-import  # noqa



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# job_titles.py
"""Collection of English and German job titles."""
import bs4
import json
import requests
import unicodedata

from bs4 import BeautifulSoup


urls = [
    'https://www.nachrichten.at/wirtschaft/karriere/tipps/bewerbung/das-steckt-hinter-englischen-berufsbezeichnungen;art215506,3298147',  # noqa
    'https://www.xing.com/campus/de/job-search',
    'https://zety.com/blog/job-titles',
]


def job_titles() -> dict:
    JOB_TITLES = {}
    try:
        with open('./persontitles/data/german_jobtitles.txt', mode='r', encoding='utf-8') as fin:  # noqa
            german_jobtitles = fin.read().split('\n')
            JOB_TITLES["German"] = german_jobtitles
        with open('./persontitles/data/english_jobtitles.txt', mode='r', encoding='utf-8') as fin:  # noqa
            english_jobtitles = fin.read().split('\n')
            JOB_TITLES["English"] = english_jobtitles
    except FileNotFoundError:
        german_jobtitles, english_jobtitles = job_titles_mix()
        with open('./persontitles/data/german_jobtitles.txt', mode='a', encoding='utf-8') as fout:  # noqa
            fout.write('\n'.join(item for item in german_jobtitles))
        with open('./persontitles/data/english_jobtitles.txt', mode='a', encoding='utf-8') as fout:  # noqa
            fout.write('\n'.join(item for item in english_jobtitles))

    return JOB_TITLES


def job_titles_mix() -> list:
    titles_1 = titles_url_1()
    titles_2 = titles_url_2()
    english_jobtitles = english_job_titles()
    german_jobtitles = [ttle for ttle in set(titles_1) | set(titles_2)]  # noqa

    return german_jobtitles, english_jobtitles


def get_soup(url) -> bs4.element.NavigableString:
    data = requests.get(url)
    soup = BeautifulSoup(data.text, 'lxml')

    return soup


def titles_url_1() -> list:
    url = urls[0]
    soup = get_soup(url)
    lines = []
    for p in soup.find_all('p'):
        lines.append(p.get_text(strip=True))

    lines = lines[5:87]
    title_collection = []
    for line in lines:
        titles = line.split('-')
        for title in titles:
            title_collection.append(title.strip())

    ttle_collection = title_collection[:]
    title_collection = []
    for title in ttle_collection:
        titles = title.split('/')
        for ttle in titles:
            title_collection.append(ttle.strip())

    ttle_collection = title_collection[:]
    title_collection = []
    for title in ttle_collection:
        titles = title.split(',')
        for ttle in titles:
            if ttle == 'Consultat':
                ttle = 'Consultant'
            elif 'Worker' in ttle:
                pass
            title_collection.append(ttle.strip())

    ttle_collection = title_collection[:]
    title_collection = []
    for title in ttle_collection:
        ttle = title.split('(')[0]
        title_collection.append(ttle.strip())

    title_collection = normalize_titles(title_collection)
    title_collection.remove('Head of...')

    return title_collection


def titles_url_2() -> list:
    titles = []
    url = urls[1]
    soup = get_soup(url)
    rawJ = soup.find('script')

    # this ignores the json stuff and only makes use of splitting strings
    J = str(rawJ)
    J1 = J.split('var _env=')[-1]
    J2 = J1.split(';')[4]
    J3 = J2.split('var REDUX_STATE=')[-1]

    for line in J3.split(','):
        if line.startswith('"name"'):
            title = line.split(':')[-1]
            title = title[1:].split('"')[0]
            if title not in ['Arbeiter', 'Absolvent', 'Satz']:
                titles.append(title)

    return titles


def english_job_titles() -> list:
    titles = []
    url = urls[2]
    soup = get_soup(url)

    # take a peek at the structure of those nested json dicts:
    # https://stackoverflow.com/a/63151716/6597765
    #     for i, rawJ in enumerate(soup.find_all('script', type="application/ld+json")):  # noqa
    #         print("index:", i)
    #         data = json.loads(rawJ.string)
    #         for k, v in data.items():
    #             print("key:", k)
    #             print("value")
    #             pprint(v)
    #             print()

    rawJs = soup.find_all('script', type='application/ld+json')
    J1 = json.loads(rawJs[1].string)
    J2 = J1['@graph'][2]
    J3 = J2['articleBody']
    titles_ = J3.split('&amp;nbsp;')
    for i, title in enumerate(titles_):
        if i not in [17, 35, 42, 51, 58, 66, 74, 81, 85, 92, 98, 103, 108, 113, 118, 123, 128, 133, 138, 144, 149, 152, 156, 159, 162, 165, 169, 172, 175, 179]:  # noqa
            titles.append(title.strip())

    titles_ = []
    for title in titles:
        title = title.split('\n')
        for ttle in title:
            titles_.append(ttle)

    titles = titles_[2:]
    titles_ = []
    titles_ = normalize_titles(titles)
    extracted_titles = extract_titles(titles_)
    titles = [title for title in set(extracted_titles)]

    for i, title in enumerate(sorted(titles)):
        print(i, title)

    return titles


def normalize_titles(titles) -> list:
    normalized_titles = []
    for title in titles:
        # print(title)
        title = unicodedata.normalize('NFKD', title.strip())
        title = title.strip()
        # print(title)
        if title not in normalized_titles:
            normalized_titles.append(title)

    return normalized_titles


def extract_titles(titles_):
    extracted_titles = []

    for title in titles_:
        if "Resume" in title:
            pass
        elif "resume" in title:
            pass
        elif "guide:" in title:
            pass
        elif "Positions" in title:
            pass
        elif "Key Takeaway" in title:
            pass
        elif "job" in title:
            pass
        elif title.startswith("Top"):
            pass
        elif title.startswith("450"):
            pass
        elif title.endswith("!"):
            pass
        elif title.endswith("?"):
            pass
        elif title.endswith(";"):
            pass
        elif title.endswith("."):
            pass
        elif title.endswith(":"):
            pass
        elif title.endswith("Jobs"):
            pass
        elif title.endswith("Sample"):
            pass
        elif title.endswith("Titles"):
            pass
        elif title in ["", "A", "Head", "Lead"]:
            pass
        elif '&amp;mdash;' in title:
            extracted_titles.append(title.split('&amp;mdash;')[0])
            extracted_titles.append(title.split('&amp;mdash;')[-1])
        elif '/' in title:
            extracted_titles.append(title.split('/')[0].strip())
            extracted_titles.append(title.split('/')[-1].strip())
        elif '(' in title:
            extracted_titles.append(title.split('(')[0].strip())
            extracted_titles.append(title.split('(')[-1][:-1].strip())
        elif '&amp;rsquo;' in title:
            title = title.replace('&amp;rsquo;', "'")
            extracted_titles.append(title.strip())
        elif '&amp;amp;' in title:
            extracted_titles.append(title.split('&amp;amp;')[0].strip())
            extracted_titles.append(title.split('&amp;amp;')[-1].strip())
        elif " or " in title:
            or_titles = []
            if title.split()[-1] in ["Cleaner", "Producer"]:
                or_titles.append("Vehicle Cleaner")
                or_titles.append("Equipment Cleaner")
                or_titles.append("Film Producer")
                or_titles.append("Video Producer")
            elif title.endswith("Taxis"):
                or_titles.append("Dispatcher for Taxis")
                or_titles.append("Dispatcher for Trucks")
            else:
                or_titles.append("Caretaker")
                or_titles.append("House Sitter")
            for ttle in or_titles:
                extracted_titles.append(ttle)
                extracted_titles.append(ttle)
        else:
            extracted_titles.append(title)

    return extracted_titles


if __name__ == '__main__':
    JOB_TITLES = job_titles()
    for i, title in enumerate(sorted(JOB_TITLES["German"])):
        print(i, title)
#     for i, title in enumerate(sorted(JOB_TITLES["English"])):
#         print(i, title)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# test_uk.py
"""Tests for academic_uk module."""
import os

from context import academic_uk


def test_filenotfound():
    os.remove('./persontitles/data/academic_uk.txt')
    ACADEMIC = academic_uk.degrees_uk()
    assert isinstance(ACADEMIC, list)


def test_academic_is_set():

    ACADEMIC = academic_uk.degrees_uk()
    assert isinstance(ACADEMIC, list)


def test_degree_in_list():

    ACADEMIC = academic_uk.degrees_uk()
    assert 'MBA' in ACADEMIC
    assert 'BSc(HealthSc)' in ACADEMIC
    assert 'Dr. e. h' not in ACADEMIC



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# rel_titles.py
"""Collection of religious titles."""
import bs4
import requests
import unicodedata
from bs4 import BeautifulSoup


def rel_titles() -> list:
    try:
        with open('./persontitles/religious_titles.txt', mode='r', encoding='utf-8') as fin:  # noqa
            REL_TITLES = fin.read().split('\n')
    except FileNotFoundError:
        REL_TITLES = religious_titles()
        with open('./persontitles/religious_titles.txt', mode='a', encoding='utf-8') as fout:  # noqa
            fout.write('\n'.join(item for item in REL_TITLES))

    return REL_TITLES


def religious_titles() -> list:
    soup = get_soup()
    lines = get_less_lines(soup)

    return lines


def get_soup() -> bs4.element.NavigableString:
    data = requests.get('https://de.wikipedia.org/wiki/Liste_religi%C3%B6ser_Amts-_und_Funktionsbezeichnungen')  # noqa
    soup = BeautifulSoup(data.text, 'lxml')

    return soup


def get_less_lines(soup):
    lines = []
    for li in soup.find_all('li'):
        a = li.find('a')
        try:
            lines.append(a.get_text())
            lines.append(li.get_text())
        except AttributeError:
            pass

    counter = 0
    for i, line in enumerate(lines):
        if line.startswith('Papst'):
            counter = i
            break
    lines = lines[counter:]

    counter = 0
    for i, line in enumerate(lines):
        if line.startswith('Panchen Rinpoche'):
            counter = i
            break
    lines = lines[:counter + 1]

    return lines


def normalize_degrees(degrees) -> list:
    normalized_degrees = []
    for degree in degrees:
        degree = unicodedata.normalize('NFKD', degree.strip())
        degree = degree.strip()
        if degree not in normalized_degrees:
            normalized_degrees.append(degree)

    return normalized_degrees


if __name__ == '__main__':
    titles = religious_titles()
    for i, title in enumerate(titles):
        print(i, title)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# peer_titles.py
"""Collection of peer titles."""
import json
import os
import requests
import sys
import unicodedata

from bs4 import BeautifulSoup
from typing import List

# PACKAGE_PARENT = '..'
# SCRIPT_DIR = os.path.dirname(
#     os.path.realpath(os.path.join(os.getcwd(), os.path.expanduser(__file__))),
# )  # isort:skip # noqa # pylint: disable=wrong-import-position
# sys.path.append(
#     os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)),
# )  # isort: skip # noqa # pylint: disable=wrong-import-position


def peertitles() -> dict:
    try:
        with open('./persontitles/data/peertitles.json', mode='r', encoding='utf-8') as fin:  # noqa
            PEERTITLES = json.load(fin)
        print("opening at ./data/...")
    except FileNotFoundError:
        PEERTITLES = _titles()
        print("saving at ./data/...")
        with open('./persontitles/data/peertitles.json', mode='w', encoding='utf-8') as fout:  # noqa
            json.dump(PEERTITLES, fout)

    return PEERTITLES


def _titles() -> dict:
    # data = requests.get('https://www.heraldik-wiki.de/wiki/Adelstitel#Adelstitel_in_verschiedenen_Sprachen')  # noqa
    peer_dict = dict()
    data = requests.get('https://de.wikipedia.org/wiki/Adelstitel')
    soup = BeautifulSoup(data.text, 'lxml')
    tbody = get_table(soup)
    for language in LANGUAGES:
        titles = get_titles(tbody, language)
        stripped_titles = strip_titles(titles)
        final_titles = finalize_titles(stripped_titles, language)
        unique_titles = _unique(final_titles)
        peer_dict[language] = unique_titles

    return peer_dict


def get_table(soup):
    for h2 in soup.find_all('h2'):
        if 'Adelstitel in verschiedenen Sprachen' in h2.text:
            tbody = h2.find_next('tbody')

    return tbody


def get_titles(tbody, language):
    """Get country specific titles."""
    counter = 16
    index = 17
    titles = []
    residue = _residue(language)
    for td in tbody.find_all('td'):
        if index % counter == residue:
            title = td.text.split(',')
            # print(title)
            titles.append(title)
        index += 1

    return titles


def _residue(language):
    language_dict = {
        'German': 1,
        'Latin': 2,
        'French': 3,
        'Italian': 4,
        'Spanish': 5,
        'English': 6,
        'Danish': 7,
        'Greek': 8,
        'Dutch': 9,
        'Czech': 10,
        'Hungarian': 11,
        'Russian': 12,
        'Persian': 13,
        'Arab': 14,
        'Chinese': 15,
        'Amharic': 0,
    }

    return language_dict[language]


LANGUAGES = [
    'German',
    'Latin',
    'French',
    'Italian',
    'Spanish',
    'English',
    'Danish',
    'Greek',
    'Dutch',
    'Czech',
    'Hungarian',
    'Russian',
    'Persian',
    'Arab',
    'Chinese',
    'Amharic',
]


def strip_titles(titles) -> list:
    peers = []
    for i, title in enumerate(titles):
        for item in title:
            if '[' in item:
                ttle = item.split('[')[0]
                peers.append(ttle)
            else:
                item = item.strip()
                if ';' in item:
                    item = item[:-1]
                peers.append(item.strip())

    return peers


def finalize_titles(peers, language) -> list:
    final_titles: List = []
    for peer in peers:
        if language == 'German':
            final_titles = finalize_german_titles(final_titles, peer)
        elif language == 'Latin':
            final_titles = finalize_latin_titles(final_titles, peer)
        elif language in ['French', 'Russian', 'Chinese', 'Persian', 'Arab']:
            final_titles = finalize_french_titles(final_titles, peer)
        elif language in ['Spanish', 'Italian', 'Danish', 'Greek', 'Hungarian']:  # noqa
            final_titles = finalize_spanish_titles(final_titles, peer)
        elif language in ['Dutch', 'Czech', 'Amharic']:
            final_titles = finalize_dutch_titles(final_titles, peer)
        elif language == 'English':
            final_titles = finalize_english_titles(final_titles, peer)

    return final_titles


def _unique(final_titles) -> list:
    uniques = []
    for item in set(final_titles):
        item = unicodedata.normalize('NFKD', item)
        uniques.append(item)

    return uniques


def finalize_german_titles(final_titles, peer):
    # German
    if peer == 'BaronFreifrau':
        final_titles.append('Baron')
        final_titles.append('Freifrau')
    elif peer == '(Groß-)Herzogin':
        final_titles.append('Großherzogin')
    elif peer in ['Herr', 'Frau', 'Fräulein']:
        pass
    elif 'Freiin' in peer:
        final_titles.append('Baronin')
        final_titles.append('Freiin')
    else:
        final_titles.append(peer)

    return final_titles


def finalize_latin_titles(final_titles, peer):
    if 'Eques/' in peer:
        final_titles.append('Eques')
        final_titles.append('Miles')
    elif '(Vir)' in peer:
        final_titles.append('Nobilis')
        final_titles.append('Vir nobilis')
    elif '(Zwei' in peer:
        final_titles.append('Imperatrix')
    elif peer == '':
        pass
    else:
        final_titles.append(peer)

    return final_titles


def finalize_french_titles(final_titles, peer):
    # print(peer)
    if 'im Sinne' in peer or 'Fürst' in peer:
        pass
    elif 'Le/' in peer:
        peer_ttle = peer.split(' ')[-1]
        if peer_ttle == 'Rougrave':
            final_titles.append('Le ' + peer_ttle)
            final_titles.append('La ' + peer_ttle)
            peer_ttle = 'Raugraveoder'
            final_titles.append('Le ' + peer_ttle)
            final_titles.append('La ' + peer_ttle)
    elif peer == '':
        pass
    elif '(' in peer:
        peer_tmp = peer.split('(')[0].strip()
        if peer_tmp == '':
            pass
        else:
            final_titles.append(peer_tmp)
        peer = peer.split('(')[-1].strip()
        peer_tmp = peer.split(')')[0].strip()
        final_titles.append(peer_tmp)
    elif ')' in peer:
        final_titles.append(peer.split(')')[0].strip())
    elif 'pl.' in peer:
        final_titles.append(peer.split('pl.')[-1].strip())
    elif peer.split(' ')[-1] == 'Schah':
        final_titles.append(peer.split(' ')[0].strip())
        final_titles.append(peer.split(' ')[-1].strip())
    else:
        final_titles.append(peer)

    return final_titles


def finalize_spanish_titles(final_titles, peer):
    if peer == '':
        pass
    elif '/' in peer:
        peers = peer.split('/')
        final_titles.append(peers[0].strip())
        final_titles.append(peers[-1].strip())
    else:
        final_titles.append(peer)

    return final_titles


def finalize_dutch_titles(final_titles, peer):
    # print(peer)
    if peer == '':
        pass
    elif 'Wildgraaf' in peer:
        final_titles.append('Wildgraaf')
        final_titles.append('Wildgraavin')
    elif ';' in peer:
        final_titles.append(peer.split(';')[0].strip())
        final_titles.append(peer.split(';')[-1].strip())
    elif '(' in peer:
        final_titles.append(peer.split('(')[0].strip())
    elif 'Grazmach' in peer:
        final_titles.append('Grazmach')
    else:
        final_titles.append(peer)

    return final_titles


def finalize_english_titles(final_titles, peer):
    if 'Baron /' in peer:
        final_titles.append('Sovereign Baron')
        final_titles.append('Sovereign Baroness')
    elif '/' in peer:
        peers = peer.split('/')
        final_titles.append(peers[0].strip())
        final_titles.append(peers[-1].strip())
#     elif 'Marchioness/' in peer:
#         final_titles.append('Marchioness')
#         final_titles.append('Margravine')
#     elif 'Knight/' in peer:
#         final_titles.append('Knight')
#         final_titles.append('Dame')
    elif peer == '':
        pass
    else:
        final_titles.append(peer)

    return final_titles


if __name__ == '__main__':
    peers = peertitles()
    for k, v in peers.items():
        print(k, v)
        print()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# gov_jobs.py
"""Collection of (currently) German government job titles."""
import bs4
import requests
import unicodedata
from bs4 import BeautifulSoup
# from pprint import pprint


urls = [
    'http://www.besoldungstabelle.de/besoldungsgruppen_amtsbezeichnungen_besoldungstabelle'  # noqa
]


def gov_jobs() -> list:
    try:
        with open('./persontitles/data/gov_jobs.txt', mode='r', encoding='utf-8') as fin:  # noqa
            GOV_JOBS = fin.read().split('\n')
    except FileNotFoundError:
        GOV_JOBS = gov_job_titles()
        with open('./persontitles/data/gov_jobs.txt', mode='a', encoding='utf-8') as fout:  # noqa
            fout.write('\n'.join(item for item in GOV_JOBS))

    return GOV_JOBS


def gov_job_titles() -> list:
    titles_1 = titles_url_1()
    print('titles_1')
    print(titles_1)
    job_titles = [ttle for ttle in set(titles_1)]  # noqa
    job_titles = change_first_word_to_female(job_titles)
    fem_male_collection = change_2_or_more_words_to_female(job_titles)

    return fem_male_collection


def get_soup(url) -> bs4.element.NavigableString:
    data = requests.get(url)
    soup = BeautifulSoup(data.text, 'lxml')

    return soup


def titles_url_1() -> list:
    url = urls[0]
    soup = get_soup(url)
    lines = []
    for p in soup.find_all('p'):
        lines.append(p.get_text(strip=True))
    print(lines)

    lines = lines[12:47]
    title_collection = []
    for line in lines:
        title = line.split(':')[-1]
        if '-Besoldung' not in title:
            title_collection.append(title.strip())

    ttle_collection = title_collection[:]
    title_collection = []
    for title in ttle_collection:
        titles = title.split(',')
        for ttle in titles:
            title_collection.append(ttle.strip())

    ttle_collection = title_collection[:]
    title_collection = []
    for title in ttle_collection:
        title = title.split('(')[0]
        title_collection.append(title.strip())

    ttle_collection = title_collection[:]
    title_collection = []
    for title in ttle_collection:
        if 'usw.' in title:
            title = title.split('usw.')[0]
        elif ' einer' in title:
            title = title.split(' einer')[0]
        elif 'ehem.' in title:
            title = title.split('ehem.')[-1]
        elif ' und' in title:
            title = title.split(' und')[0]
        title_collection.append(title.strip())

    title_collection = normalize_titles(title_collection)

    return title_collection


def normalize_titles(titles) -> list:
    normalized_titles = []
    for title in titles:
        title = unicodedata.normalize('NFKD', title.strip())
        title = title.strip()
        if title not in normalized_titles:
            normalized_titles.append(title)

    return normalized_titles


def change_first_word_to_female(titles) -> list:
    female_vers_added = []
    for title in titles:
        female_vers_added.append(title.strip())
        fields = title.split(' ')
        end = ' '.join(fields[1:])
        ttle = fields[0]
        fem_title = change_to_female_variant(ttle) + ' ' + end
        while '  ' in fem_title:
            fem_title = fem_title.replace('  ', ' ')
        fem_title = fem_title.strip()
        if fem_title not in female_vers_added:
            female_vers_added.append(fem_title)

    return female_vers_added


def change_to_female_variant(title) -> str:
    # need to normalize with NFC because otherwise "Präsident" will not be
    # recognized and even counted with 10 letters!
    ttle = unicodedata.normalize('NFC', title.strip())
    if ttle in ['Direktor', 'Konsul', 'Präsident', 'Chef', 'Kanzler', 'Sekretär']:  # noqa
        fem_title = ttle + 'in '
    elif ttle.endswith('ektor') or ttle.endswith('onsul') or\
        ttle.endswith('räsident') or ttle.endswith('hef') or\
        ttle.endswith('anzler') or ttle.endswith('ekretär') or\
        ttle.endswith('rofessor') or ttle.endswith('fleger') or\
        ttle.endswith('eister') or ttle.endswith('ührer') or\
        ttle.endswith('ommissar') or ttle.endswith('rinär') or\
        ttle.endswith('theker') or ttle.endswith('konservator') or\
        ttle.endswith('schafter') or ttle.endswith('stent'):  # noqa
        fem_title = ttle + 'in '
    elif ttle.endswith('rat'):
        fem_title = ttle[:-3] + 'rätin'
    elif ttle.endswith('arzt'):
        fem_title = ttle[:-4] + 'ärztin'
    elif ttle.endswith('walt'):
        fem_title = ttle[:-4] + 'wältin'
    elif ttle == 'Rat':
        fem_title = 'Rätin '
    elif ttle == 'Arzt':
        fem_title = 'Ärztin '
    else:
        fem_title = title

    return fem_title


def change_2_or_more_words_to_female(titles) -> list:
    female_vers_added = []
    for title in titles:
        # append the male versions and the female versions so far
        female_vers_added.append(title)

        fields = title.split(' ')
        if len(fields) > 1:
            male_ttle = fields[-1]
            male_ttle = unicodedata.normalize('NFC', male_ttle.strip())
            fem_ttle = change_to_female_variant(male_ttle)
            if male_ttle != fem_ttle:
                fem_title = change_to_female(fields) + ' ' + fem_ttle.strip()
                female_vers_added.append(fem_title)

    return female_vers_added


def change_to_female(fields) -> str:
    fields_wo_last_field = fields[:-1]
    fem_words = ''
    for field in fields_wo_last_field:
        if field.endswith('r'):
            field = field[:-1]
            fem_words = fem_words + field + ' '

    return fem_words.strip()


if __name__ == '__main__':
    titles = gov_job_titles()
    for i, title in enumerate(sorted(titles)):
        print(i, title)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# test_job_titles.py
"""Tests for academic degrees."""
import os
import sys

PACKAGE_PARENT = '..'
SCRIPT_DIR = os.path.dirname(
    os.path.realpath(os.path.join(os.getcwd(), os.path.expanduser(__file__))),
)  # isort:skip # noqa # pylint: disable=wrong-import-position
sys.path.append(
    os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)),
)  # isort: skip # noqa # pylint: disable=wrong-import-position

from context import job_titles  # noqa


def test_no_file():
    try:
        os.remove('./persontitles/data/job_titles.txt')
    except FileNotFoundError:
        pass
    JOB_TITLES = job_titles.job_titles()
    assert isinstance(JOB_TITLES, dict)


def test_titles_is_dict_of_lists():
    JOB_TITLES = job_titles.job_titles()
    assert isinstance(JOB_TITLES, dict)
    assert isinstance(JOB_TITLES["German"], list)
    assert isinstance(JOB_TITLES["English"], list)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# academic_us.py
"""Collection of US academic degrees."""
import unicodedata

import requests
from bs4 import BeautifulSoup


def degrees_us():

    try:
        with open('./persontitles/data/academic_us.txt', mode='r', encoding='utf-8') as fin:  # noqa
            ACADEMIC = fin.read().split('\n')
    except FileNotFoundError:
        ACADEMIC = []
        ACADEMIC_1 = academic_degrees_1()
        degrees_1 = get_degrees_1(ACADEMIC_1)
        ACADEMIC_2 = academic_degrees_2()
        degrees_2 = get_degrees_2(ACADEMIC_2)
        degrees_2 = revise_degrees_2(degrees_2)
        abbrevs = _degrees()

        DEGREES = abbrevs + degrees_1 + degrees_2
        for abbr in DEGREES:
            abbr = unicodedata.normalize('NFKD', abbr)
            ACADEMIC.append(abbr)

        with open('./persontitles/data/academic_us.txt', mode='a', encoding='utf-8') as fout:  # noqa
            fout.write('\n'.join(item for item in set(ACADEMIC)))

    return set(ACADEMIC)


def academic_degrees_1() -> list:
    # collected from here:
    # https://www.google.com/url?sa=t&rct=j&q=&esrc=s&source=web&cd=&ved=2ahUKEwih6eSh_uHsAhXRzqQKHXOuChAQFjAAegQIBBAC&url=https%3A%2F%2Fwww2.ed.gov%2Fabout%2Foffices%2Flist%2Fous%2Finternational%2Fusnei%2Fus%2Fmaster.doc&usg=AOvVaw3dwR4LfY5S4eN2jcGrkCA-
    # https://www.google.com/url?sa=t&rct=j&q=&esrc=s&source=web&cd=&cad=rja&uact=8&ved=2ahUKEwih6eSh_uHsAhXRzqQKHXOuChAQFjABegQIBhAC&url=https%3A%2F%2Fwww2.ed.gov%2Fabout%2Foffices%2Flist%2Fous%2Finternational%2Fusnei%2Fus%2Fbachelor.doc&usg=AOvVaw22CyjpUN5Z639pCUEqPpfu
    # https://www.google.com/url?sa=t&rct=j&q=&esrc=s&source=web&cd=&ved=2ahUKEwiTiqrK_-HsAhUG_KQKHRW_AL0QFjACegQIBBAC&url=https%3A%2F%2Fwww2.ed.gov%2Fabout%2Foffices%2Flist%2Fous%2Finternational%2Fusnei%2Fus%2Fdoctorate.doc&usg=AOvVaw2PGDESFOCKJpbUol7vRaFq
    with open('./persontitles/data/academic_us1.txt', mode='r', encoding='utf-8') as fin:  # noqa
        ACADEMIC_1 = fin.read().split('\n')
    return ACADEMIC_1


def academic_degrees_2() -> list:
    with open('./persontitles/data/academic_us2.txt', mode='r', encoding='utf-8') as fin:  # noqa
        ACADEMIC_2 = fin.read().split('\n')
    return ACADEMIC_2


def get_degrees_1(ACADEMIC_1):
    degrees = []
    for degree in ACADEMIC_1:
        degree = degree.split('-')[0].strip()
        if len(degree.split('. ')) > 1:
            degree = degree.split('. ')[0]
        if ' or ' in degree:
            for ttle in degree.split(' or '):
                degrees.append(ttle.strip())
        elif len(degree) < 3:
            pass
        else:
            degrees.append(degree.strip())
    return degrees


def get_degrees_2(ACADEMIC_2):
    degrees = []
    for i, degree in enumerate(ACADEMIC_2):
        degree_brackets = degree.split('(')
        for degr in degree_brackets:
            if ')' in degr:
                abbr = degr.split(')')[0].strip()
                degrees.append(abbr)
    return degrees


def revise_degrees_2(degrees):
    rev_degrees = []
    for i, degree in enumerate(degrees):
        if '/' in degree:
            abbrevs = degree.split('/')
            for abbr in abbrevs:
                rev_degrees.append(abbr.strip())
        elif ',' in degree:
            abbrevs = degree.split(',')
            for abbr in abbrevs:
                if 'or ' in abbr:
                    abbr = abbr.split('or ')[-1].strip()
                rev_degrees.append(abbr.strip())
        elif ' or ' in degree:
            abbrevs = degree.split(' or ')
            for abbr in abbrevs:
                rev_degrees.append(abbr.strip())
        else:
            rev_degrees.append(degree)

    return rev_degrees


def get_degrees(soup):
    degrees = []
    script = soup.find_all('li')
    for line in script:
        strgline = str(line)
        splitline = strgline.split('-')
        if '.' in splitline[0][:8]:
            degrees.append(splitline[0].split('<li>')[-1])

    return degrees


def final_degrees(abbrevs):
    final_degrees = []
    for degree in abbrevs:
        if 'F&amp;TV' in degree:
            degree = 'B.F&TV.'
            final_degrees.append(degree)
        elif ' or ' in degree:
            abbrs = degree.split(' or ')
            for abbr in abbrs:
                final_degrees.append(abbr.strip())
        elif '</li>' in degree:
            degree = degree.split(' ')[0]
            final_degrees.append(degree.strip())
        else:
            final_degrees.append(degree.strip())
    return final_degrees


def _degrees():
    data = requests.get('https://abbreviations.yourdictionary.com/articles/degree-abbreviations.html')  # noqa
    soup = BeautifulSoup(data.text, 'lxml')
    degrees = get_degrees(soup)
    abbrevs = final_degrees(degrees)

    return abbrevs


if __name__ == '__main__':
    try:
        with open('./data/academic_us.txt') as fin:
            ACADEMIC = fin.read().split('\n')
            for i, degree in enumerate(ACADEMIC):
                print(i, degree)
    except FileNotFoundError:
        ACADEMIC_US = degrees_us()
        for i, degree in enumerate(ACADEMIC_US):
            print(i, degree)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# academic_degrees.py
"""Collection of academic degrees."""
import json
import os
import sys

PACKAGE_PARENT = '..'
SCRIPT_DIR = os.path.dirname(
    os.path.realpath(os.path.join(os.getcwd(), os.path.expanduser(__file__))),
)  # isort:skip # noqa # pylint: disable=wrong-import-position
sys.path.append(
    os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)),
)  # isort: skip # noqa # pylint: disable=wrong-import-position

from persontitles.academic_german import degrees_ger  # noqa
from persontitles.academic_german import german_abbrevs  # noqa
from persontitles.academic_uk import degrees_uk  # noqa
from persontitles.academic_us import degrees_us  # noqa


def degrees() -> dict:
    try:
        with open('./persontitles/data/degrees.json', mode='r', encoding='utf-8') as fin:  # noqa
            DEGREES = json.load(fin)
    except FileNotFoundError:
        DEGREES = collect_degrees()
        with open('./persontitles/data/degrees.json', mode='w', encoding='utf-8') as fout:  # noqa
            json.dump(DEGREES, fout)

    return DEGREES


def collect_degrees():
    DEGREES = dict()
    degrees = []

    for degree in degrees_ger():
        degrees.append(degree)
    DEGREES['D'] = degrees

    degrees = []
    degrees_d = degrees_ger()
    for degree in german_abbrevs(degrees_d):
        degrees.append(degree)
    DEGREES['german_abbrevs'] = degrees

    degrees = []
    for degree in degrees_uk():
        degrees.append(degree)
    DEGREES['UK'] = degrees

    degrees = []
    for degree in degrees_us():
        degrees.append(degree)
    DEGREES['US'] = degrees

    return DEGREES


if __name__ == '__main__':
    DEGREES = collect_degrees()
    for k, v in DEGREES.items():
        print(k)
        print(v)
        print()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# test_degrees.py
"""Tests for academic degrees."""
import os
import sys

PACKAGE_PARENT = '..'
SCRIPT_DIR = os.path.dirname(
    os.path.realpath(os.path.join(os.getcwd(), os.path.expanduser(__file__))),
)  # isort:skip # noqa # pylint: disable=wrong-import-position
sys.path.append(
    os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)),
)  # isort: skip # noqa # pylint: disable=wrong-import-position

from context import academic_degrees  # noqa


def test_no_file():
    try:
        os.remove('./persontitles/data/degrees.json')
    except FileNotFoundError:
        pass
    ACADEMIC = academic_degrees.degrees_ger()
    assert isinstance(ACADEMIC, list)


def test_degrees_is_dict():
    DEGREES = academic_degrees.degrees()
    assert isinstance(DEGREES, dict)


def test_keys():
    DEGREES = academic_degrees.degrees()
    for k, v in DEGREES.items():
        assert k in ['D', 'UK', 'US', 'german_abbrevs']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# academic_austria.py
"""Collection of German academic degrees."""
import unicodedata

import requests
from bs4 import BeautifulSoup


def degrees_austria() -> set:
    try:
        with open('./persontitles/data/academic_austrian.txt', mode='r', encoding='utf-8') as fin:  # noqa
            ACADEMIC = fin.read().split('\n')
    except FileNotFoundError:
        degrees = _degrees()
        ACADEMIC = []
        for abbr in degrees:
            ACADEMIC.append(abbr)
        with open('./persontitles/data/academic_austrian.txt', mode='a', encoding='utf-8') as fout:  # noqa
            fout.write('\n'.join(item for item in set(ACADEMIC)))

    return set(ACADEMIC)


def get_degrees(soup):
    degrees = []
    for tr in soup.find_all('tr'):
        values = [td.text for td in tr.find_all('td')]
        try:
            degrees.append(values[0])
        except IndexError:
            # print("IndexError:", values)
            pass
    return degrees


def strip_degrees(degrees) -> list:
    abbrevs = []
    for degree in degrees:
        if '(' in degree:
            bracket_content = degree.split('(')[-1]
            if 'Master' in degree:
                degree = bracket_content.split(')')[0]
                abbrevs.append(degree)
            elif 'FH' in bracket_content:
                degree = degree.split(')')[0]
                degree = degree + ')'
                abbrevs.append(degree)
            elif 'Med.' in bracket_content:
                degree = degree.split(')')[0]
                degree = degree + ')'
                abbrevs.append(degree)
            elif 't.o.' in bracket_content:
                degree = degree.split(')')[0]
                degree = degree + ')'
                abbrevs.append(degree)
            else:
                degree = degree.split('(')[0]
                if '/' in degree:
                    for ttle in degree.split('/'):
                        abbrevs.append(ttle)
                else:
                    abbrevs.append(degree)
        elif len(set(degree)) == 1:
            pass
        elif 'oder' in degree:
            for ttle in degree.split('oder'):
                abbrevs.append(ttle.strip())
        elif '/' in degree:
            for ttle in degree.split('/'):
                if ttle == 'Kunstvermittlung':
                    abbrevs.append('Dipl. Kunstpädagogik/Kunstvermittlung')
                else:
                    abbrevs.append(ttle.strip())
        elif '[' in degree:
            abbrevs.append(degree.split('[')[0])
        else:
            abbrevs.append(degree)

    return abbrevs


def final_degrees(abbrevs):
    final_degrees = []
    for abbr in abbrevs:
        # get rid of unicode's \xa0 for the empty spaces
        # https://stackoverflow.com/a/34669482/6597765
        abbr = unicodedata.normalize('NFKD', abbr)
        if len(set(abbr)) == 1:
            pass
        elif '[' in abbr:
            abbr = abbr.split('[')[0]
            final_degrees.append(abbr.strip())
        elif '...' in abbr:
            pass
        else:
            final_degrees.append(abbr.strip())
    return final_degrees


def _degrees():
    urls = ['https://www.oesterreich.gv.at/themen/leben_in_oesterreich/titel_und_auszeichnungen/1/1/Seite.1730512.html',  # noqa
            'https://www.oesterreich.gv.at/themen/leben_in_oesterreich/titel_und_auszeichnungen/1/1/Seite.1730513.html',  # noqa
            'https://www.oesterreich.gv.at/themen/leben_in_oesterreich/titel_und_auszeichnungen/1/1/Seite.1730514.html',  # noqa
            'https://www.oesterreich.gv.at/themen/leben_in_oesterreich/titel_und_auszeichnungen/1/3/Seite.1730515.html',  # noqa
            'https://www.oesterreich.gv.at/themen/leben_in_oesterreich/titel_und_auszeichnungen/1/3/Seite.1730516.html',  # noqa
            'https://www.oesterreich.gv.at/themen/leben_in_oesterreich/titel_und_auszeichnungen/1/3/Seite.1730522.html',  # noqa
            'https://www.oesterreich.gv.at/themen/leben_in_oesterreich/titel_und_auszeichnungen/1/4/Seite.1730517.html',  # noqa
            'https://www.oesterreich.gv.at/themen/leben_in_oesterreich/titel_und_auszeichnungen/1/4/Seite.1730518.html',  # noqa
            'https://www.oesterreich.gv.at/themen/leben_in_oesterreich/titel_und_auszeichnungen/1/Seite.1730507.html',  # noqa
            'https://www.oesterreich.gv.at/themen/leben_in_oesterreich/titel_und_auszeichnungen/1/2.html',  # noqa
            'https://www.oesterreich.gv.at/themen/leben_in_oesterreich/titel_und_auszeichnungen/1/2/Seite.1730519.html',  # noqa
            'https://www.oesterreich.gv.at/themen/leben_in_oesterreich/titel_und_auszeichnungen/1/Seite.1730505.html',  # noqa
            'https://www.oesterreich.gv.at/themen/leben_in_oesterreich/titel_und_auszeichnungen/1/Seite.1730508.html'  # noqa
            ]
    data = requests.get(urls[0])  # noqa
    soup = BeautifulSoup(data.text, 'lxml')
    degrees = get_degrees(soup)
    abbrevs = strip_degrees(degrees)
    degrees = final_degrees(abbrevs)

    return degrees


if __name__ == '__main__':
    ACADEMIC = degrees_austria()
    for i, degree in enumerate(ACADEMIC):
        print(i, degree)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# test_german.py
"""Tests for academic_german module."""
import os
import sys

PACKAGE_PARENT = '..'
SCRIPT_DIR = os.path.dirname(
    os.path.realpath(os.path.join(os.getcwd(), os.path.expanduser(__file__))),
)  # isort:skip # noqa # pylint: disable=wrong-import-position
sys.path.append(
    os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)),
)  # isort: skip # noqa # pylint: disable=wrong-import-position

from context import academic_german_wiki  # noqa


def test_filenotfound():
    os.remove('./persontitles/data/academic_german_wiki.txt')
    ACADEMIC = academic_german_wiki.degrees_ger_wiki()
    assert isinstance(ACADEMIC, list)


def test_academic_is_list():

    ACADEMIC = academic_german_wiki.degrees_ger_wiki()
    assert isinstance(ACADEMIC, list)


def test_degree_in_list():

    ACADEMIC = academic_german_wiki.degrees_ger_wiki()
    assert 'MBA' in ACADEMIC
    assert 'Dr. e. h.' in ACADEMIC
    assert 'Dr. e. h' not in ACADEMIC



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup  # type: ignore

with open('README.rst') as readme_file:
    long_description = readme_file.read()

requirements = [
    'beautifulsoup4',
    'lxml',
    'requests',
]

test_requirements = [
    'pytest>=3',
]

setup(
    author='Oliver Stapel',
    author_email='hardy.ecc95@gmail.com',
    python_requires='>=3.7',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Operating System :: POSIX :: Linux',
        'Topic :: Other/Nonlisted Topic',
    ],
    description='Collections of academic degrees, job (and peer) titles, and salutations.',  # noqa
    install_requires=requirements,
    license='MIT license',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    include_package_data=True,
    package_data={"persontitles": ["*.txt", "*.json"]},
    keywords='person, titles, degrees',
    name='persontitles',
    # packages=find_packages(exclude=('tests',)),  # noqa
    packages=['persontitles'],
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/0LL13/persontitles',
    version='0.1.6',
    zip_safe=False,
    extras_require={
        'dev': ['check-manifest'],
        'test': ['pytest-cov'],
    },
    project_urls={
        'Bug Reports': 'https://github.com/0LL13/persontitles/issues',
        'Source': 'https://github.com/0LL13/persontitles',
    },
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# test_german.py
"""Tests for combined module."""
import os
import sys

PACKAGE_PARENT = '..'
SCRIPT_DIR = os.path.dirname(
    os.path.realpath(os.path.join(os.getcwd(), os.path.expanduser(__file__))),
)  # isort:skip # noqa # pylint: disable=wrong-import-position
sys.path.append(
    os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)),
)  # isort: skip # noqa # pylint: disable=wrong-import-position

from context import academic_german  # noqa


def test_no_file():
    os.remove('./persontitles/data/academic_german.txt')
    ACADEMIC = academic_german.degrees_ger()
    assert isinstance(ACADEMIC, list)


def test_academic_is_list():
    ACADEMIC = academic_german.degrees_ger()
    assert isinstance(ACADEMIC, list)


def test_degree_in_list():
    ACADEMIC = academic_german.degrees_ger()
    assert 'MBA' in ACADEMIC
    assert 'Dipl.-Inf.' in ACADEMIC
    assert 'Dr. e. h' not in ACADEMIC


def test_no_empty_space():
    DEGREES_W_SPACES = ['Dipl.-Ing. agr.', 'B. A.', 'lic. theol. säusel', 'Dr. med.']  # noqa
    DEGREES_WO_SPACES = academic_german.no_empty_space_in_degree(DEGREES_W_SPACES)  # noqa
    for degree in DEGREES_WO_SPACES:
        assert ' ' not in degree


def test_add_empty_space():
    DEGREES_WO_SPACES = ['Dipl.-Ing.agr.', 'B.A.', 'lic.theol.säusel', 'Dr.med.']  # noqa
    DEGREES_W_SPACES = academic_german.add_empty_space(DEGREES_WO_SPACES)
    for degree in DEGREES_W_SPACES:
        assert ' ' in degree


def test_handle_nones():
    DEGREES_WO_SPACES = ['Dr.']  # noqa
    DEGREES_W_SPACES = academic_german.add_empty_space(DEGREES_WO_SPACES)
    for degree in DEGREES_W_SPACES:
        assert degree != ''


def test_handle_no_dots():
    DEGREES_WO_SPACES = ['MBA']  # noqa
    DEGREES_W_SPACES = academic_german.add_empty_space(DEGREES_WO_SPACES)
    for degree in DEGREES_W_SPACES:
        assert degree != ''


def test_find_abbrevs():
    DEGREES = ['Dipl.-Ing. agr.', 'B. A.', 'lic. theol. säusel', 'Dr. med.']  # noqa
    abbrevs = academic_german.german_abbrevs(DEGREES)
    assert abbrevs == ['Dipl.-Ing.', 'agr.', 'B.', 'A.', 'lic.', 'theol.',
                       'säusel', 'Dr.', 'med.']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# test_peertitles.py
"""Tests for peertitles."""
import os

from context import peertitles


def test_peertitles_is_dict():
    os.remove('./persontitles/data/peertitles.json')
    PEERTITLES = peertitles.peertitles()
    assert isinstance(PEERTITLES, dict)


def test_keys():
    LANGUAGES = [
        'German',
        'Latin',
        'French',
        'Italian',
        'Spanish',
        'English',
        'Danish',
        'Greek',
        'Dutch',
        'Czech',
        'Hungarian',
        'Russian',
        'Persian',
        'Arab',
        'Chinese',
        'Amharic',
    ]

    PEERTITLES = peertitles.peertitles()
    for k, v in PEERTITLES.items():
        assert k in LANGUAGES



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# academic_uk.py
"""Collection of UK academic degrees."""
import unicodedata

import requests
from bs4 import BeautifulSoup


def degrees_uk() -> list:
    try:
        with open('./persontitles/data/academic_uk.txt', mode='r', encoding='utf-8') as fin:  # noqa
            DEGREES = fin.read().split('\n')
    except FileNotFoundError:
        DEGREES = uk_degrees()
        with open('./persontitles/data/academic_uk.txt', mode='a', encoding='utf-8') as fout:  # noqa
            fout.write('\n'.join(item for item in DEGREES))
    return DEGREES


def uk_degrees():
    data = requests.get('https://en.wikipedia.org/wiki/British_degree_abbreviations')  # noqa
    soup = BeautifulSoup(data.text, 'lxml')
    lines = get_lines(soup)
    uk_degrees = []
    for i, degree in enumerate(lines):
        if i > 19 and i < 440:
            uk_degrees.append(degree)
    abbrevs = strip_degrees(uk_degrees)
    fnl_degrees = final_degrees(abbrevs)
    degrees = normalize_degrees(fnl_degrees)

    return degrees


def get_lines(soup):
    lines = []
    for li in soup.find_all('li'):
        values = [li.get_text(strip=True)]
        lines.append(values[0])
    return lines


def strip_degrees(degrees) -> list:
    abbrevs = []
    for i, degree in enumerate(degrees):
        abbr = degree.split('-')[0].strip()
        abbrevs.append(abbr.strip())
    return abbrevs


def final_degrees(abbrevs):
    final_degrees = []
    for abbr in abbrevs:
        if ' or ' in abbr:
            abbrs = abbr.split('or')
            for ab in abbrs:
                final_degrees.append(ab.strip())
        elif ',' in abbr:
            abbrs = abbr.split(',')
            for ab in abbrs:
                final_degrees.append(ab.strip())
        elif ') ' in abbr:
            abbrs = abbr.split(') ')
            ab = abbrs[0].strip()
            if ab[-1] != ')':
                ab = ab + ')'
            final_degrees.append(ab)
        else:
            final_degrees.append(abbr.strip())

    return final_degrees


def normalize_degrees(degrees):
    DEGREES = []
    for abbr in set(degrees):
        abbr = unicodedata.normalize('NFKD', abbr)
        DEGREES.append(abbr)

    return DEGREES


if __name__ == '__main__':
    ACADEMIC_UK = degrees_uk()
    for i, degree in enumerate(ACADEMIC_UK):
        print(i, degree)
